local ADDON_NAME, addon = ...
_G[ADDON_NAME] = addon

-- initialize addon table
addon.events = addon.events or {}

local events = addon.events
local profile

local profile_default = {
	anchor = "CENTER",
	x = 0,
	y = -175,
}

function addon:Initialize()
	SLASH_PlayerSpeed1 = "/playerspeed"
	SLASH_PlayerSpeed2 = "/ps"
	SlashCmdList.PlayerSpeed = function(msg) self:CommandHandler(strlower(msg)) end

	self.enabled = false
	self.updateFreq = 1 / 10
end

function addon:CreateProfile()
	local isNew
	local profileName = GetRealmName().."-"..UnitName("player")
	if type(PlayerSpeedDB) ~= "table" then
		PlayerSpeedDB = {}
	end
	if not PlayerSpeedDB[profileName] then
		PlayerSpeedDB[profileName] = {}
		isNew = true
	end
	profile = PlayerSpeedDB[profileName]
	return isNew
end

function addon:CreateWidgets()
	if self.frame then return end

	local frame = CreateFrame("Frame", nil, _G.UIParent, "BackdropTemplate")
	frame:ClearAllPoints()
	frame:SetPoint(profile_default.anchor, profile_default.x, profile_default.y)
	frame:SetSize(46, 22)
	frame:SetBackdrop({
		bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
		edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
		edgeSize = 14,
		tile = false,
		insets = {left = 3, right = 3, top = 3, bottom = 3}
	})
	frame:SetBackdropColor(0, 0, 0, 0.7)

	function frame:OnMouseDown(button)
		if button == "RightButton" then
			self:StopMovingOrSizing()
			self:EnableMouse(false)
		else
			self:StartMoving()
		end
	end

	function frame:OnMouseUp(button)
		self:StopMovingOrSizing()
	end

	--function frame:OnUpdate(elapsed)
	--	addon:Update(elapsed)
	--end

	function frame:OnEvent(event, ...)
		events[event](addon, event, ...)
	end

	frame:SetScript("OnMouseDown", frame.OnMouseDown)
	frame:SetScript("OnMouseUp", frame.OnMouseUp)
	frame:SetScript("OnDragStop", frame.OnMouseUp)
	frame:SetScript("OnEvent", frame.OnEvent)

	frame:SetMovable(true)
	frame:EnableMouse(false)

	frame:UnregisterAllEvents()
	for event, func in pairs(events) do
		if type(func) == "function" then frame:RegisterEvent(event) end
	end

	local text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")
	text:ClearAllPoints()
	text:SetPoint("CENTER", frame, 0, 0)
	text:SetText("---%")
	frame.text = text

	self.frame = frame
	--self.text = text
end

do
	local select, floor, strconcat = select, math.floor, strconcat
	local GetUnitSpeed, IsFlying, IsFalling, IsSwimming = GetUnitSpeed, IsFlying, IsFalling, IsSwimming

	local sincelast = 0
	local movementType = 2 -- 1:unknown 2:run 3:flight 4:swim
	local UNITNAME = "player"
	local TEXT_HEADER, TEXT_PERCENT, TEXT_FOOTER = "|cffff0000", "%", "|r"
	function addon:Update(elapsed) -- self is addon.frame
		sincelast = sincelast + elapsed
		if sincelast < self.updateFreq then return end
		sincelast = 0
		if IsFlying() then
			movementType = 3
		elseif movementType == 4 and IsFalling() then -- jumping on the surface of water
			return
		else
			movementType = IsSwimming() and 4 or 2
		end
		local speed = floor(select(movementType, GetUnitSpeed(UNITNAME)) / 7 * 100 + 0.5)
		if (movementType == 2 and speed < 100) or (movementType == 4 and speed < 67) then
			self.text:SetText(strconcat(TEXT_HEADER, speed, TEXT_PERCENT, TEXT_FOOTER))
		else
			self.text:SetText(strconcat(speed, TEXT_PERCENT))
		end
	end
end

local function Profile_Set(key, value)
	local changed = false
	if type(profile_default[key]) == "number" and type(value) == "number" then
		changed = math.floor(math.abs(profile_default[key]) * 1000) ~= math.floor(math.abs(value) * 1000)
	else
		changed = profile_default[key] ~= value
	end
	profile[key] = changed and value or nil
end

local function Profile_Get(key)
	return profile[key] or profile_default[key]
end

function addon:SaveProfile()
	local anchor, _, _, x, y = self.frame:GetPoint(1)
	Profile_Set("anchor", anchor)
	Profile_Set("x", x)
	Profile_Set("y", y)
end

function addon:LoadProfile()
	local anchor = Profile_Get("anchor")
	local x = Profile_Get("x")
	local y = Profile_Get("y")
	self.frame:ClearAllPoints()
	self.frame:SetPoint(anchor, x, y)
end

function addon:SetEnabled(value)
	if self.enabled == value then return end
	self.enabled = value
	if value then
		--self.frame:SetScript("OnUpdate", self.frame.OnUpdate)
		self.frame.updateFreq = self.updateFreq
		self.frame:SetScript("OnUpdate", self.Update)
	else
		self.frame:SetScript("OnUpdate", nil)
	end
end

function events:ADDON_LOADED(event, name)
	if name ~= ADDON_NAME then return end
	self:Initialize()
	local isNew = self:CreateProfile()
	if isNew then
		self.frame:EnableMouse(true)
		self.CommandHandler()
	end
end

function events:PLAYER_ENTERING_WORLD(event)
	self:LoadProfile()
	self:SetEnabled(true)
end

function events:PLAYER_LEAVING_WORLD(event)
	self:SaveProfile()
	self:SetEnabled(false)
end

function addon:CommandHandler(msg)
	if msg == "lock" then
		self.frame:EnableMouse(false)
		self:SaveProfile()
	elseif msg == "unlock" then
		self.frame:EnableMouse(true)
	elseif msg == "reset" then
		self.frame:ClearAllPoints()
		self.frame:SetPoint(profile_default.anchor, profile_default.x, profile_default.y)
		self.frame:EnableMouse(true)
	else
		DEFAULT_CHAT_FRAME:AddMessage(
			format("|cff%s%s|r v%s", "ff7f3f", ADDON_NAME, GetAddOnMetadata(ADDON_NAME, "Version"))..
			"\n/ps lock\n/ps unlock\n/ps reset"
		)
	end
end

do
	addon:CreateWidgets()
end
